import uuid
import enum
from sqlalchemy import Column, String, TIMESTAMP, Text, Enum as SAEnum, ForeignKey
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import relationship
from sqlalchemy.sql import func
from ..account_base import AccountBase

class ProjectStatusEnum(enum.Enum):
    PLANNED = "PLANNED"
    IN_PROGRESS = "IN_PROGRESS"
    COMPLETED = "COMPLETED"
    FAILED = "FAILED"

class Project(AccountBase):
    __tablename__ = "project"

    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    record_id = Column(UUID(as_uuid=True), ForeignKey("record.id", ondelete="CASCADE"), nullable=False)
    prompt = Column(Text)
    conversation_history = Column(Text)
    status = Column(SAEnum(ProjectStatusEnum), nullable=False, default=ProjectStatusEnum.PLANNED)
    notes = Column(Text)

    created_at = Column(TIMESTAMP(timezone=True), nullable=False, server_default=func.current_timestamp())
    updated_at = Column(TIMESTAMP(timezone=True), nullable=False, server_default=func.current_timestamp(), onupdate=func.current_timestamp())

    record = relationship("Record", back_populates="projects")
    project_steps = relationship("ProjectStep", back_populates="project", cascade="all, delete-orphan")
    outputs = relationship("ProjectOutput", back_populates="project", cascade="all, delete-orphan")


import uuid
import enum
from sqlalchemy import Column, String, TIMESTAMP, Text, Enum as SAEnum, ForeignKey
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import relationship
from sqlalchemy.sql import func
from ..account_base import AccountBase
from .project_step_data_source_association import project_step_data_source_association
from .project_step_tool_association import project_step_tool_association

class ProjectStepStatusEnum(enum.Enum):
    PLANNED = "PLANNED"
    IN_PROGRESS = "IN_PROGRESS"
    COMPLETED = "COMPLETED"
    FAILED = "FAILED"

class ProjectStep(AccountBase):
    __tablename__ = "project_step"

    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    project_id = Column(UUID(as_uuid=True), ForeignKey("project.id", ondelete="CASCADE"), nullable=False)
    title = Column(String(255), nullable=False)
    description = Column(Text)
    prompt = Column(Text)
    output_name = Column(String(255))
    status = Column(SAEnum(ProjectStepStatusEnum), nullable=False, default=ProjectStepStatusEnum.PLANNED)
    notes = Column(Text)

    created_at = Column(TIMESTAMP(timezone=True), nullable=False, server_default=func.current_timestamp())
    updated_at = Column(TIMESTAMP(timezone=True), nullable=False, server_default=func.current_timestamp(), onupdate=func.current_timestamp())

    project = relationship("Project", back_populates="project_steps")
    output = relationship("ProjectOutput", back_populates="project_step", uselist=False, cascade="all, delete-orphan")
    
    data_sources = relationship("DataSource", secondary=project_step_data_source_association)
    tools = relationship("ToolUseSpec", secondary=project_step_tool_association)


from sqlalchemy.ext.asyncio import AsyncSession
from leodb.models.account.project_output import ProjectOutput
from leodb.schemas.project_output import ProjectOutputCreate, ProjectOutputUpdate, ProjectOutputSchema
from .base_repository import BaseRepository

class ProjectOutputRepository(BaseRepository[ProjectOutput, ProjectOutputCreate, ProjectOutputUpdate, ProjectOutputSchema]):
    
    def __init__(self, db: AsyncSession, model: type[ProjectOutput]):
        super().__init__(db, model, ProjectOutputSchema)


from typing import Optional
from uuid import UUID

from sqlalchemy import select
from sqlalchemy.ext.asyncio import AsyncSession

from leodb.models.general.user_preferences import UserPreferences
from leodb.schemas.user_preferences import UserPreferencesCreate, UserPreferencesUpdate, UserPreferencesSchema
from .base_repository import BaseRepository

class UserPreferencesRepository(BaseRepository[UserPreferences, UserPreferencesCreate, UserPreferencesUpdate, UserPreferencesSchema]):

    def __init__(self, db: AsyncSession, model: type[UserPreferences]):
        super().__init__(db, model, UserPreferencesSchema)

    async def get_by_user_id(self, *, user_id: UUID) -> Optional[UserPreferencesSchema]:
        statement = select(self.model).where(self.model.user_id == user_id)
        result = await self.db.execute(statement)
        db_obj = result.scalar_one_or_none()
        return self.read_schema.from_orm(db_obj) if db_obj else None

    async def create_for_user(self, *, user_id: UUID, obj_in: UserPreferencesCreate) -> UserPreferencesSchema:
        create_data = obj_in.model_dump()
        create_data["user_id"] = user_id
        
        db_obj = self.model(**create_data)
        self.db.add(db_obj)
        await self.db.flush()
        await self.db.refresh(db_obj)
        return self.read_schema.from_orm(db_obj)


from sqlalchemy.ext.asyncio import AsyncSession
from leodb.models.account.project_step import ProjectStep
from leodb.schemas.project_step import ProjectStepCreate, ProjectStepUpdate, ProjectStepSchema
from .base_repository import BaseRepository

class ProjectStepRepository(BaseRepository[ProjectStep, ProjectStepCreate, ProjectStepUpdate, ProjectStepSchema]):
    
    def __init__(self, db: AsyncSession, model: type[ProjectStep]):
        super().__init__(db, model, ProjectStepSchema)


from typing import Optional
from uuid import UUID

from sqlalchemy import select
from sqlalchemy.ext.asyncio import AsyncSession

from leodb.models.general.package import Package
from leodb.schemas.billing import SubscriptionSchema
from .base_repository import BaseRepository

class PackageRepository(BaseRepository[Package, None, None, SubscriptionSchema]):

    def __init__(self, db: AsyncSession, model: type[Package]):
        super().__init__(db, model, SubscriptionSchema)

    async def get_active_for_account(self, *, account_id: UUID) -> Optional[SubscriptionSchema]:
        statement = (
            select(self.model)
            .where(self.model.account_id == account_id)
            .where(self.model.is_active == True)
        )
        result = await self.db.execute(statement)
        db_obj = result.scalar_one_or_none()
        return self.read_schema.from_orm(db_obj) if db_obj else None


from typing import Any, Dict, Generic, List, Optional, Type, TypeVar, Union
from uuid import UUID
from sqlalchemy import select
from sqlalchemy.ext.asyncio import AsyncSession
from pydantic import BaseModel
from leodb.models.base import Base

ModelType = TypeVar("ModelType", bound=Base)
CreateSchemaType = TypeVar("CreateSchemaType", bound=BaseModel)
UpdateSchemaType = TypeVar("UpdateSchemaType", bound=BaseModel)
ReadSchemaType = TypeVar("ReadSchemaType", bound=BaseModel)

class BaseRepository(Generic[ModelType, CreateSchemaType, UpdateSchemaType, ReadSchemaType]):
    def __init__(self, db: AsyncSession, model: Type[ModelType], read_schema: Type[ReadSchemaType]):
        """
        Base class for data repositories.

        :param db: The SQLAlchemy async session
        :param model: A SQLAlchemy model class
        :param read_schema: A Pydantic schema for reading/returning data
        """
        self.db = db
        self.model = model
        self.read_schema = read_schema

    async def get(self, id: Any) -> Optional[ReadSchemaType]:
        from sqlalchemy.inspection import inspect
        primary_key_name = inspect(self.model).primary_key[0].name
        statement = select(self.model).where(getattr(self.model, primary_key_name) == id)
        result = await self.db.execute(statement)
        db_obj = result.scalar_one_or_none()
        return self.read_schema.from_orm(db_obj) if db_obj else None

    async def get_multi(
        self, *, skip: int = 0, limit: int = 100
    ) -> List[ReadSchemaType]:
        statement = select(self.model).offset(skip).limit(limit)
        result = await self.db.execute(statement)
        db_objs = result.scalars().all()
        return [self.read_schema.from_orm(db_obj) for db_obj in db_objs]

    async def create(self, *, obj_in: CreateSchemaType) -> ReadSchemaType:
        obj_in_data = obj_in.model_dump()
        db_obj = self.model(**obj_in_data)
        self.db.add(db_obj)
        await self.db.flush()
        await self.db.refresh(db_obj)
        return self.read_schema.from_orm(db_obj)

    async def update(
        self, *, db_obj: ModelType, obj_in: Union[UpdateSchemaType, Dict[str, Any]]
    ) -> ReadSchemaType:
        if isinstance(obj_in, dict):
            update_data = obj_in
        else:
            update_data = obj_in.model_dump(exclude_unset=True)
        
        for field, value in update_data.items():
            setattr(db_obj, field, value)
            
        self.db.add(db_obj)
        await self.db.flush()
        await self.db.refresh(db_obj)
        return self.read_schema.from_orm(db_obj)

    async def remove(self, *, id: UUID) -> Optional[ReadSchemaType]:
        # First, get the object to be returned later
        statement = select(self.model).where(getattr(self.model, "id", None) == id)
        result = await self.db.execute(statement)
        db_obj = result.scalar_one_or_none()

        if db_obj:
            # Convert to schema before deletion
            read_obj = self.read_schema.from_orm(db_obj)
            await self.db.delete(db_obj)
            await self.db.flush()
            return read_obj
        return None


from sqlalchemy.ext.asyncio import AsyncSession
from leodb.models.general.account import Account
from leodb.schemas.account import AccountCreate, AccountUpdate, AccountSchema
from .base_repository import BaseRepository

class AccountRepository(BaseRepository[Account, AccountCreate, AccountUpdate, AccountSchema]):
    def __init__(self, db: AsyncSession, model: type[Account]):
        super().__init__(db, model, AccountSchema)


"""Create general schema tables

Revision ID: 5ddbaa686ea6
Revises: 
Create Date: 2025-06-19 14:13:34.907163

"""
from typing import Sequence, Union

from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision: str = '5ddbaa686ea6'
down_revision: Union[str, None] = None
branch_labels: Union[str, Sequence[str], None] = None
depends_on: Union[str, Sequence[str], None] = None


def upgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_table('accounts',
    sa.Column('account_id', sa.UUID(), nullable=False),
    sa.Column('account_name', sa.String(length=255), nullable=True),
    sa.Column('is_team', sa.Boolean(), nullable=False),
    sa.Column('config_admin_email', sa.String(), nullable=True),
    sa.Column('created_at', sa.TIMESTAMP(timezone=True), server_default=sa.text('CURRENT_TIMESTAMP'), nullable=False),
    sa.Column('updated_at', sa.TIMESTAMP(timezone=True), server_default=sa.text('CURRENT_TIMESTAMP'), nullable=False),
    sa.PrimaryKeyConstraint('account_id'),
    schema='general'
    )
    op.create_index('idx_accounts_created_at', 'accounts', ['created_at'], unique=False, schema='general')
    op.create_index(op.f('ix_general_accounts_is_team'), 'accounts', ['is_team'], unique=False, schema='general')
    op.create_table('account_data_connections',
    sa.Column('id', sa.UUID(), nullable=False),
    sa.Column('account_id', sa.UUID(), nullable=False),
    sa.Column('db_host', sa.String(), nullable=True),
    sa.Column('db_port', sa.Integer(), nullable=True),
    sa.Column('db_name', sa.String(), nullable=True),
    sa.Column('db_user', sa.String(), nullable=True),
    sa.Column('db_password', sa.String(), nullable=True),
    sa.Column('vector_store_url', sa.String(), nullable=True),
    sa.Column('vector_store_key', sa.String(), nullable=True),
    sa.ForeignKeyConstraint(['account_id'], ['general.accounts.account_id'], ),
    sa.PrimaryKeyConstraint('id'),
    schema='general'
    )
    op.create_index(op.f('ix_general_account_data_connections_account_id'), 'account_data_connections', ['account_id'], unique=False, schema='general')
    op.create_table('invoices',
    sa.Column('invoice_id', sa.UUID(), nullable=False),
    sa.Column('account_id', sa.UUID(), nullable=False),
    sa.Column('invoice_number', sa.String(length=50), nullable=False),
    sa.Column('date_from', sa.TIMESTAMP(timezone=True), nullable=False),
    sa.Column('date_to', sa.TIMESTAMP(timezone=True), nullable=False),
    sa.Column('invoice_date', sa.TIMESTAMP(timezone=True), server_default=sa.text('CURRENT_TIMESTAMP'), nullable=False),
    sa.Column('due_date', sa.TIMESTAMP(timezone=True), nullable=False),
    sa.Column('package', sa.String(length=20), nullable=False),
    sa.Column('plan_type', sa.String(length=20), nullable=False),
    sa.Column('nb_licences', sa.String(length=10), nullable=False),
    sa.Column('subtotal', sa.Numeric(precision=10, scale=2), nullable=False),
    sa.Column('tax1_rate', sa.Numeric(precision=5, scale=4), nullable=False),
    sa.Column('tax1_amount', sa.Numeric(precision=10, scale=2), nullable=False),
    sa.Column('tax2_rate', sa.Numeric(precision=5, scale=4), nullable=False),
    sa.Column('tax2_amount', sa.Numeric(precision=10, scale=2), nullable=False),
    sa.Column('total', sa.Numeric(precision=10, scale=2), nullable=False),
    sa.Column('currency', sa.String(length=3), nullable=False),
    sa.Column('payment_status', sa.String(length=20), nullable=False),
    sa.Column('payment_date', sa.TIMESTAMP(timezone=True), nullable=True),
    sa.Column('payment_confirmation_no', sa.String(length=100), nullable=True),
    sa.Column('payment_method_used', sa.String(length=50), nullable=True),
    sa.Column('notes', sa.String(length=1000), nullable=True),
    sa.Column('metadata', sa.JSON(), nullable=True),
    sa.Column('created_at', sa.TIMESTAMP(timezone=True), server_default=sa.text('CURRENT_TIMESTAMP'), nullable=False),
    sa.Column('updated_at', sa.TIMESTAMP(timezone=True), server_default=sa.text('CURRENT_TIMESTAMP'), nullable=False),
    sa.CheckConstraint("package IN ('STANDARD', 'PRO', 'ELITE')", name='invoices_package_type_check'),
    sa.CheckConstraint("payment_status IN ('pending', 'paid', 'failed', 'cancelled', 'refunded')", name='invoices_payment_status_check'),
    sa.CheckConstraint("plan_type IN ('individual', 'team')", name='invoices_plan_type_check'),
    sa.CheckConstraint('date_to > date_from', name='invoices_date_range_check'),
    sa.CheckConstraint('due_date >= invoice_date', name='invoices_due_date_check'),
    sa.CheckConstraint('subtotal >= 0 AND total >= 0', name='invoices_amounts_positive_check'),
    sa.ForeignKeyConstraint(['account_id'], ['general.accounts.account_id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('invoice_id'),
    schema='general'
    )
    op.create_index('idx_invoices_period', 'invoices', ['date_from', 'date_to'], unique=False, schema='general')
    op.create_index(op.f('ix_general_invoices_account_id'), 'invoices', ['account_id'], unique=False, schema='general')
    op.create_index(op.f('ix_general_invoices_date_from'), 'invoices', ['date_from'], unique=False, schema='general')
    op.create_index(op.f('ix_general_invoices_date_to'), 'invoices', ['date_to'], unique=False, schema='general')
    op.create_index(op.f('ix_general_invoices_due_date'), 'invoices', ['due_date'], unique=False, schema='general')
    op.create_index(op.f('ix_general_invoices_invoice_number'), 'invoices', ['invoice_number'], unique=True, schema='general')
    op.create_index(op.f('ix_general_invoices_payment_confirmation_no'), 'invoices', ['payment_confirmation_no'], unique=False, schema='general')
    op.create_index(op.f('ix_general_invoices_payment_date'), 'invoices', ['payment_date'], unique=False, schema='general')
    op.create_index(op.f('ix_general_invoices_payment_status'), 'invoices', ['payment_status'], unique=False, schema='general')
    op.create_table('packages',
    sa.Column('package_id', sa.UUID(), nullable=False),
    sa.Column('account_id', sa.UUID(), nullable=False),
    sa.Column('package', sa.String(length=20), nullable=False),
    sa.Column('begin_date', sa.TIMESTAMP(timezone=True), server_default=sa.text('CURRENT_TIMESTAMP'), nullable=False),
    sa.Column('end_date', sa.TIMESTAMP(timezone=True), nullable=True),
    sa.Column('is_active', sa.Boolean(), nullable=False),
    sa.Column('created_at', sa.TIMESTAMP(timezone=True), server_default=sa.text('CURRENT_TIMESTAMP'), nullable=False),
    sa.Column('updated_at', sa.TIMESTAMP(timezone=True), server_default=sa.text('CURRENT_TIMESTAMP'), nullable=False),
    sa.CheckConstraint("package IN ('STANDARD', 'PRO', 'ELITE')", name='packages_package_type_check'),
    sa.CheckConstraint('end_date IS NULL OR end_date > begin_date', name='packages_date_range_check'),
    sa.ForeignKeyConstraint(['account_id'], ['general.accounts.account_id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('package_id'),
    schema='general'
    )
    op.create_index(op.f('ix_general_packages_account_id'), 'packages', ['account_id'], unique=False, schema='general')
    op.create_index(op.f('ix_general_packages_begin_date'), 'packages', ['begin_date'], unique=False, schema='general')
    op.create_index(op.f('ix_general_packages_is_active'), 'packages', ['is_active'], unique=False, schema='general')
    op.create_index(op.f('ix_general_packages_package'), 'packages', ['package'], unique=False, schema='general')
    op.create_table('payment_methods',
    sa.Column('payment_method_id', sa.UUID(), nullable=False),
    sa.Column('account_id', sa.UUID(), nullable=False),
    sa.Column('stripe_account_id', sa.String(), nullable=True),
    sa.Column('stripe_payment_method_id', sa.String(), nullable=True),
    sa.Column('card_holder_name', sa.String(), nullable=True),
    sa.Column('card_last_four', sa.String(length=4), nullable=True),
    sa.Column('card_brand', sa.String(length=20), nullable=False),
    sa.Column('card_exp_month', sa.String(length=2), nullable=False),
    sa.Column('card_exp_year', sa.String(length=4), nullable=False),
    sa.Column('is_default', sa.Boolean(), nullable=False),
    sa.Column('is_active', sa.Boolean(), nullable=False),
    sa.Column('is_expired', sa.Boolean(), nullable=False),
    sa.Column('stripe_metadata', sa.JSON(), nullable=True),
    sa.Column('created_at', sa.TIMESTAMP(timezone=True), server_default=sa.text('CURRENT_TIMESTAMP'), nullable=False),
    sa.Column('updated_at', sa.TIMESTAMP(timezone=True), server_default=sa.text('CURRENT_TIMESTAMP'), nullable=False),
    sa.Column('last_used_at', sa.TIMESTAMP(timezone=True), nullable=True),
    sa.CheckConstraint("card_brand IN ('visa', 'mastercard', 'amex', 'discover', 'diners', 'jcb', 'unionpay', 'unknown')", name='payment_methods_card_brand_check'),
    sa.ForeignKeyConstraint(['account_id'], ['general.accounts.account_id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('payment_method_id'),
    schema='general'
    )
    op.create_index('idx_payment_methods_is_expired', 'payment_methods', ['is_expired'], unique=False, schema='general')
    op.create_index(op.f('ix_general_payment_methods_account_id'), 'payment_methods', ['account_id'], unique=False, schema='general')
    op.create_index(op.f('ix_general_payment_methods_is_active'), 'payment_methods', ['is_active'], unique=False, schema='general')
    op.create_index(op.f('ix_general_payment_methods_is_default'), 'payment_methods', ['is_default'], unique=False, schema='general')
    op.create_index(op.f('ix_general_payment_methods_is_expired'), 'payment_methods', ['is_expired'], unique=False, schema='general')
    op.create_index(op.f('ix_general_payment_methods_last_used_at'), 'payment_methods', ['last_used_at'], unique=False, schema='general')
    op.create_table('users',
    sa.Column('user_id', sa.UUID(), nullable=False),
    sa.Column('entra_id', sa.String(length=255), nullable=False),
    sa.Column('tenant_id', sa.String(length=255), nullable=True),
    sa.Column('account_id', sa.UUID(), nullable=False),
    sa.Column('email', sa.String(), nullable=True),
    sa.Column('display_name', sa.String(), nullable=True),
    sa.Column('given_name', sa.String(), nullable=True),
    sa.Column('family_name', sa.String(), nullable=True),
    sa.Column('preferred_username', sa.String(), nullable=True),
    sa.Column('config_delegated_to_name', sa.String(), nullable=True),
    sa.Column('config_delegated_to_email', sa.String(), nullable=True),
    sa.Column('is_active', sa.Boolean(), nullable=False),
    sa.Column('is_team_admin', sa.Boolean(), nullable=False),
    sa.Column('status', sa.Enum('ACTIVE', 'PENDING_TEAM_INVITATION', 'INACTIVE', name='user_status_enum'), nullable=False),
    sa.Column('metadata', sa.JSON(), nullable=True),
    sa.Column('created_at', sa.TIMESTAMP(timezone=True), server_default=sa.text('CURRENT_TIMESTAMP'), nullable=False),
    sa.Column('last_login_at', sa.TIMESTAMP(timezone=True), nullable=True),
    sa.Column('updated_at', sa.TIMESTAMP(timezone=True), server_default=sa.text('CURRENT_TIMESTAMP'), nullable=False),
    sa.ForeignKeyConstraint(['account_id'], ['general.accounts.account_id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('user_id'),
    schema='general'
    )
    op.create_index('idx_users_last_login', 'users', ['last_login_at'], unique=False, schema='general')
    op.create_index(op.f('ix_general_users_account_id'), 'users', ['account_id'], unique=False, schema='general')
    op.create_index(op.f('ix_general_users_entra_id'), 'users', ['entra_id'], unique=True, schema='general')
    op.create_index(op.f('ix_general_users_is_active'), 'users', ['is_active'], unique=False, schema='general')
    op.create_index(op.f('ix_general_users_is_team_admin'), 'users', ['is_team_admin'], unique=False, schema='general')
    op.create_index(op.f('ix_general_users_last_login_at'), 'users', ['last_login_at'], unique=False, schema='general')
    op.create_index(op.f('ix_general_users_status'), 'users', ['status'], unique=False, schema='general')
    op.create_index(op.f('ix_general_users_tenant_id'), 'users', ['tenant_id'], unique=False, schema='general')
    op.create_table('user_invitations',
    sa.Column('invitation_id', sa.UUID(), nullable=False),
    sa.Column('invited_by_user_id', sa.UUID(), nullable=True),
    sa.Column('account_id', sa.UUID(), nullable=False),
    sa.Column('invited_user_email', sa.String(), nullable=False),
    sa.Column('invited_user_id', sa.UUID(), nullable=True),
    sa.Column('token_hash', sa.String(length=255), nullable=False),
    sa.Column('status', sa.Enum('PENDING', 'ACCEPTED', 'EXPIRED', 'CANCELLED', name='invitation_status_enum'), nullable=False),
    sa.Column('expires_at', sa.DateTime(timezone=True), nullable=False),
    sa.Column('created_at', sa.DateTime(timezone=True), server_default=sa.text('now()'), nullable=False),
    sa.Column('updated_at', sa.DateTime(timezone=True), server_default=sa.text('now()'), nullable=False),
    sa.ForeignKeyConstraint(['account_id'], ['general.accounts.account_id'], ondelete='CASCADE'),
    sa.ForeignKeyConstraint(['invited_by_user_id'], ['general.users.user_id'], ondelete='SET NULL'),
    sa.ForeignKeyConstraint(['invited_user_id'], ['general.users.user_id'], ondelete='SET NULL'),
    sa.PrimaryKeyConstraint('invitation_id'),
    sa.UniqueConstraint('token_hash'),
    schema='general'
    )
    op.create_index(op.f('ix_general_user_invitations_invited_user_email'), 'user_invitations', ['invited_user_email'], unique=False, schema='general')
    op.create_index(op.f('ix_general_user_invitations_invited_user_id'), 'user_invitations', ['invited_user_id'], unique=False, schema='general')
    op.create_table('user_preferences',
    sa.Column('preference_id', sa.UUID(), nullable=False),
    sa.Column('user_id', sa.UUID(), nullable=False),
    sa.Column('language', sa.String(length=5), nullable=False),
    sa.Column('leo_call_name', sa.String(), nullable=True),
    sa.Column('agent_tone', sa.String(length=20), nullable=False),
    sa.Column('default_currency', sa.String(length=3), nullable=False),
    sa.Column('notification_email_enabled', sa.Boolean(), nullable=False),
    sa.Column('notification_sms_enabled', sa.Boolean(), nullable=False),
    sa.Column('notification_email', sa.String(), nullable=True),
    sa.Column('notification_phone', sa.String(), nullable=True),
    sa.Column('custom_settings', sa.JSON(), nullable=True),
    sa.Column('created_at', sa.TIMESTAMP(timezone=True), server_default=sa.text('CURRENT_TIMESTAMP'), nullable=False),
    sa.Column('updated_at', sa.TIMESTAMP(timezone=True), server_default=sa.text('CURRENT_TIMESTAMP'), nullable=False),
    sa.CheckConstraint("agent_tone IN ('friendly', 'professional', 'direct')", name='user_preferences_agent_tone_check'),
    sa.CheckConstraint("language IN ('fr', 'en')", name='user_preferences_language_check'),
    sa.ForeignKeyConstraint(['user_id'], ['general.users.user_id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('preference_id'),
    schema='general'
    )
    op.create_index(op.f('ix_general_user_preferences_user_id'), 'user_preferences', ['user_id'], unique=True, schema='general')
    op.create_table('user_sessions',
    sa.Column('session_id', sa.UUID(), nullable=False),
    sa.Column('user_id', sa.UUID(), nullable=False),
    sa.Column('jwt_token_hash', sa.String(length=255), nullable=False),
    sa.Column('client_type', sa.String(length=50), nullable=False),
    sa.Column('client_version', sa.String(length=50), nullable=True),
    sa.Column('client_ip_address', sa.String(), nullable=True),
    sa.Column('user_agent', sa.Text(), nullable=True),
    sa.Column('created_at', sa.TIMESTAMP(timezone=True), server_default=sa.text('CURRENT_TIMESTAMP'), nullable=False),
    sa.Column('expires_at', sa.TIMESTAMP(timezone=True), nullable=False),
    sa.Column('last_activity_at', sa.TIMESTAMP(timezone=True), server_default=sa.text('CURRENT_TIMESTAMP'), nullable=False),
    sa.Column('is_active', sa.Boolean(), nullable=False),
    sa.Column('metadata', sa.JSON(), nullable=True),
    sa.CheckConstraint("client_type IN ('WEB', 'TEAMS', 'MOBILE', 'API', 'OTHER')", name='user_sessions_client_type_check'),
    sa.ForeignKeyConstraint(['user_id'], ['general.users.user_id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('session_id'),
    schema='general'
    )
    op.create_index('idx_user_sessions_jwt_hash', 'user_sessions', ['jwt_token_hash'], unique=False, schema='general')
    op.create_index(op.f('ix_general_user_sessions_expires_at'), 'user_sessions', ['expires_at'], unique=False, schema='general')
    op.create_index(op.f('ix_general_user_sessions_is_active'), 'user_sessions', ['is_active'], unique=False, schema='general')
    op.create_index(op.f('ix_general_user_sessions_jwt_token_hash'), 'user_sessions', ['jwt_token_hash'], unique=False, schema='general')
    op.create_index(op.f('ix_general_user_sessions_user_id'), 'user_sessions', ['user_id'], unique=False, schema='general')
    op.create_table('user_themes',
    sa.Column('id', sa.UUID(), nullable=False),
    sa.Column('user_id', sa.UUID(), nullable=False),
    sa.Column('theme_key', sa.String(length=50), nullable=False),
    sa.Column('created_at', sa.TIMESTAMP(timezone=True), server_default=sa.text('CURRENT_TIMESTAMP'), nullable=False),
    sa.Column('updated_at', sa.TIMESTAMP(timezone=True), server_default=sa.text('CURRENT_TIMESTAMP'), nullable=False),
    sa.ForeignKeyConstraint(['user_id'], ['general.users.user_id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id'),
    schema='general'
    )
    op.create_index(op.f('ix_general_user_themes_user_id'), 'user_themes', ['user_id'], unique=False, schema='general')
    op.create_table('user_theme_indicators',
    sa.Column('id', sa.UUID(), nullable=False),
    sa.Column('user_theme_id', sa.UUID(), nullable=False),
    sa.Column('indicator_key', sa.String(length=50), nullable=False),
    sa.Column('created_at', sa.TIMESTAMP(timezone=True), server_default=sa.text('CURRENT_TIMESTAMP'), nullable=False),
    sa.Column('updated_at', sa.TIMESTAMP(timezone=True), server_default=sa.text('CURRENT_TIMESTAMP'), nullable=False),
    sa.ForeignKeyConstraint(['user_theme_id'], ['general.user_themes.id'], ondelete='CASCADE'),
    sa.PrimaryKeyConstraint('id'),
    schema='general'
    )
    op.create_index(op.f('ix_general_user_theme_indicators_user_theme_id'), 'user_theme_indicators', ['user_theme_id'], unique=False, schema='general')
    # ### end Alembic commands ###


def downgrade() -> None:
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f('ix_general_user_theme_indicators_user_theme_id'), table_name='user_theme_indicators', schema='general')
    op.drop_table('user_theme_indicators', schema='general')
    op.drop_index(op.f('ix_general_user_themes_user_id'), table_name='user_themes', schema='general')
    op.drop_table('user_themes', schema='general')
    op.drop_index(op.f('ix_general_user_sessions_user_id'), table_name='user_sessions', schema='general')
    op.drop_index(op.f('ix_general_user_sessions_jwt_token_hash'), table_name='user_sessions', schema='general')
    op.drop_index(op.f('ix_general_user_sessions_is_active'), table_name='user_sessions', schema='general')
    op.drop_index(op.f('ix_general_user_sessions_expires_at'), table_name='user_sessions', schema='general')
    op.drop_index('idx_user_sessions_jwt_hash', table_name='user_sessions', schema='general')
    op.drop_table('user_sessions', schema='general')
    op.drop_index(op.f('ix_general_user_preferences_user_id'), table_name='user_preferences', schema='general')
    op.drop_table('user_preferences', schema='general')
    op.drop_index(op.f('ix_general_user_invitations_invited_user_id'), table_name='user_invitations', schema='general')
    op.drop_index(op.f('ix_general_user_invitations_invited_user_email'), table_name='user_invitations', schema='general')
    op.drop_table('user_invitations', schema='general')
    op.drop_index(op.f('ix_general_users_tenant_id'), table_name='users', schema='general')
    op.drop_index(op.f('ix_general_users_status'), table_name='users', schema='general')
    op.drop_index(op.f('ix_general_users_last_login_at'), table_name='users', schema='general')
    op.drop_index(op.f('ix_general_users_is_team_admin'), table_name='users', schema='general')
    op.drop_index(op.f('ix_general_users_is_active'), table_name='users', schema='general')
    op.drop_index(op.f('ix_general_users_entra_id'), table_name='users', schema='general')
    op.drop_index(op.f('ix_general_users_account_id'), table_name='users', schema='general')
    op.drop_index('idx_users_last_login', table_name='users', schema='general')
    op.drop_table('users', schema='general')
    op.drop_index(op.f('ix_general_payment_methods_last_used_at'), table_name='payment_methods', schema='general')
    op.drop_index(op.f('ix_general_payment_methods_is_expired'), table_name='payment_methods', schema='general')
    op.drop_index(op.f('ix_general_payment_methods_is_default'), table_name='payment_methods', schema='general')
    op.drop_index(op.f('ix_general_payment_methods_is_active'), table_name='payment_methods', schema='general')
    op.drop_index(op.f('ix_general_payment_methods_account_id'), table_name='payment_methods', schema='general')
    op.drop_index('idx_payment_methods_is_expired', table_name='payment_methods', schema='general')
    op.drop_table('payment_methods', schema='general')
    op.drop_index(op.f('ix_general_packages_package'), table_name='packages', schema='general')
    op.drop_index(op.f('ix_general_packages_is_active'), table_name='packages', schema='general')
    op.drop_index(op.f('ix_general_packages_begin_date'), table_name='packages', schema='general')
    op.drop_index(op.f('ix_general_packages_account_id'), table_name='packages', schema='general')
    op.drop_table('packages', schema='general')
    op.drop_index(op.f('ix_general_invoices_payment_status'), table_name='invoices', schema='general')
    op.drop_index(op.f('ix_general_invoices_payment_date'), table_name='invoices', schema='general')
    op.drop_index(op.f('ix_general_invoices_payment_confirmation_no'), table_name='invoices', schema='general')
    op.drop_index(op.f('ix_general_invoices_invoice_number'), table_name='invoices', schema='general')
    op.drop_index(op.f('ix_general_invoices_due_date'), table_name='invoices', schema='general')
    op.drop_index(op.f('ix_general_invoices_date_to'), table_name='invoices', schema='general')
    op.drop_index(op.f('ix_general_invoices_date_from'), table_name='invoices', schema='general')
    op.drop_index(op.f('ix_general_invoices_account_id'), table_name='invoices', schema='general')
    op.drop_index('idx_invoices_period', table_name='invoices', schema='general')
    op.drop_table('invoices', schema='general')
    op.drop_index(op.f('ix_general_account_data_connections_account_id'), table_name='account_data_connections', schema='general')
    op.drop_table('account_data_connections', schema='general')
    op.drop_index(op.f('ix_general_accounts_is_team'), table_name='accounts', schema='general')
    op.drop_index('idx_accounts_created_at', table_name='accounts', schema='general')
    op.drop_table('accounts', schema='general')
    # ### end Alembic commands ###



from sqlalchemy.orm import declarative_base, DeclarativeBase
from sqlalchemy import MetaData
from leodb.core.config import settings

# Métadonnées centralisées avec le schéma par défaut
# Toutes les tables qui héritent de Base utiliseront ce schéma
metadata_obj = MetaData(schema=settings.POSTGRES_GENERAL_SCHEMA)

class Base(DeclarativeBase):
    """
    Classe de base pour tous les modèles SQLAlchemy.
    Applique un schéma par défaut à toutes les tables.
    """
    metadata = metadata_obj


import uuid
import enum
from sqlalchemy import Column, String, TIMESTAMP, Text, Enum as SAEnum, ForeignKey, Integer
from sqlalchemy.dialects.postgresql import UUID
from sqlalchemy.orm import relationship
from sqlalchemy.sql import func
from ..account_base import AccountBase
from leodb.encryption.manager import EncryptedModelMixin

class ContentTypeEnum(enum.Enum):
    PDF = "PDF"
    JSON = "JSON"
    MD = "MD"
    CSV = "CSV"

class ProjectOutput(AccountBase, EncryptedModelMixin):
    __tablename__ = "project_output"

    _encrypted_fields = ['content','notes']

    id = Column(UUID(as_uuid=True), primary_key=True, default=uuid.uuid4)
    project_id = Column(UUID(as_uuid=True), ForeignKey("project.id", ondelete="CASCADE"), nullable=False)
    project_step_id = Column(UUID(as_uuid=True), ForeignKey("project_step.id", ondelete="CASCADE"), nullable=False)
    content_type = Column(SAEnum(ContentTypeEnum), nullable=False)
    output_name = Column(String(255))
    content = Column(Text) # Encrypted
    notes = Column(Text) # Encrypted
    version_number = Column(Integer, nullable=False, default=1)

    created_at = Column(TIMESTAMP(timezone=True), nullable=False, server_default=func.current_timestamp())
    updated_at = Column(TIMESTAMP(timezone=True), nullable=False, server_default=func.current_timestamp(), onupdate=func.current_timestamp())

    project = relationship("Project", back_populates="outputs")
    project_step = relationship("ProjectStep", back_populates="output")


from datetime import datetime, timedelta
import logging
from typing import List, Optional
from uuid import UUID

from sqlalchemy import select, func, case, text
from sqlalchemy.ext.asyncio import AsyncSession

from leodb.models.account.conversation import Conversation
from leodb.models.account.message import Message
from leodb.schemas.conversation import ConversationCreate, ConversationUpdate, ConversationInfo
from .base_repository import BaseRepository

class ConversationRepository(BaseRepository[Conversation, ConversationCreate, ConversationUpdate, ConversationInfo]):

    def __init__(self, db: AsyncSession, model: type[Conversation]):
        super().__init__(db, model, ConversationInfo)
    
    async def create(self, *, obj_in: ConversationCreate) -> ConversationInfo:
        # Note: le nom du champ dans le modèle est meta_data
        db_obj = Conversation(
            user_id=obj_in.user_id,
            client_type=obj_in.client_type,
            client_version=obj_in.client_version,
            title=obj_in.title,
            meta_data=obj_in.metadata
        )
        self.db.add(db_obj)
        await self.db.flush()
        await self.db.refresh(db_obj)
        return self.read_schema.from_orm(db_obj)

    async def get_all_by_user(self, *, user_id: str, limit: int = 100) -> List[ConversationInfo]:
        """
        Lists conversations for a user, returning only their ID and title.
        """
        # [TEMP_DEBUG] Log the current search_path
        try:
            search_path_result = await self.db.execute(text("SHOW search_path;"))
            current_search_path = search_path_result.scalar_one_or_none()
            logging.warning(f"[TEMP_DEBUG] Current search_path for get_all_by_user: {current_search_path}")
        except Exception as e:
            logging.error(f"[TEMP_DEBUG] Could not retrieve search_path: {e}")

        stmt = (
            select(Conversation.conversation_id, Conversation.title)
            .where(Conversation.user_id == user_id)
            .order_by(Conversation.last_updated_at.desc())
            .limit(limit)
        )
        result = await self.db.execute(stmt)
        # Les résultats sont des tuples (id, title), on les mappe vers le schéma
        return [ConversationInfo(conversation_id=row.conversation_id, title=row.title) for row in result]

    async def update_title(self, *, conversation_id: UUID, title: str) -> Optional[ConversationInfo]:
        # We need the model object to update it, not the schema from self.get()
        statement = select(self.model).where(self.model.conversation_id == conversation_id)
        result = await self.db.execute(statement)
        db_obj = result.scalar_one_or_none()

        if db_obj:
            db_obj.title = title
            self.db.add(db_obj)
            await self.db.flush()
            await self.db.refresh(db_obj)
            return self.read_schema.from_orm(db_obj)
        return None

    async def clean_old_conversations(self, *, max_age_days: int) -> int:
        cutoff_date = datetime.utcnow() - timedelta(days=max_age_days)
        stmt = Conversation.__table__.delete().where(Conversation.last_updated_at < cutoff_date)
        result = await self.db.execute(stmt)
        await self.db.flush()
        return result.rowcount


from typing import Optional, List, Tuple, Dict, Any
from uuid import UUID
from datetime import datetime
from sqlalchemy import select, func
from sqlalchemy.ext.asyncio import AsyncSession

from leodb.models.general.user import User
from leodb.schemas.user import UserCreate, UserUpdate, UserSchema
from .base_repository import BaseRepository

class UserRepository(BaseRepository[User, UserCreate, UserUpdate, UserSchema]):
    
    def __init__(self, db: AsyncSession, model: type[User]):
        super().__init__(db, model, UserSchema)

    async def get_by_email(self, *, email: str) -> Optional[UserSchema]:
        statement = select(self.model).where(self.model.email == email)
        result = await self.db.execute(statement)
        db_obj = result.scalar_one_or_none()
        return self.read_schema.from_orm(db_obj) if db_obj else None

    async def get_by_entra_id(self, *, entra_id: str) -> Optional[UserSchema]:
        statement = select(self.model).where(self.model.entra_id == entra_id)
        result = await self.db.execute(statement)
        db_obj = result.scalar_one_or_none()
        return self.read_schema.from_orm(db_obj) if db_obj else None

    async def get_by_account_paginated(
        self, *, account_id: UUID, skip: int = 0, limit: int = 100
    ) -> Tuple[int, List[UserSchema]]:
        count_statement = select(func.count()).select_from(self.model).where(self.model.account_id == account_id)
        total = (await self.db.execute(count_statement)).scalar_one()

        statement = (
            select(self.model)
            .where(self.model.account_id == account_id)
            .offset(skip)
            .limit(limit)
        )
        result = await self.db.execute(statement)
        db_objs = result.scalars().all()
        return total, [self.read_schema.from_orm(db_obj) for db_obj in db_objs]

    async def add_to_account(self, *, user: User, account_id: UUID) -> UserSchema:
        user.account_id = account_id
        self.db.add(user)
        await self.db.flush()
        await self.db.refresh(user)
        return self.read_schema.from_orm(user)

    async def remove_from_account(self, *, user: User) -> UserSchema:
        user.account_id = None
        self.db.add(user)
        await self.db.flush()
        await self.db.refresh(user)
        return self.read_schema.from_orm(user)

    async def sync_from_azure(self, *, azure_user_data: Dict[str, Any]) -> UserSchema:
        """
        Finds a user by their entra_id and updates them, or creates them if they don't exist.
        """
        entra_id = azure_user_data.get("entra_id")
        if not entra_id:
            raise ValueError("entra_id is required in azure_user_data")

        # We need the model object to perform an update
        statement = select(self.model).where(self.model.entra_id == entra_id)
        result = await self.db.execute(statement)
        db_obj = result.scalar_one_or_none()

        if db_obj:
            # User exists, update them
            return await self.update(db_obj=db_obj, obj_in=azure_user_data)
        else:
            # User does not exist, create them
            user_in = UserCreate(**azure_user_data)
            return await self.create(obj_in=user_in)

    async def activate(self, *, user: User, activation_data: Dict[str, Any]) -> UserSchema:
        update_data = {"status": "ACTIVE", **activation_data}
        return await self.update(db_obj=user, obj_in=update_data)

    async def reject(self, *, user: User, rejection_data: Dict[str, Any]) -> UserSchema:
        update_data = {"status": "REJECTED", **rejection_data}
        return await self.update(db_obj=user, obj_in=update_data)

    async def update_last_login(self, *, user: User) -> UserSchema:
        return await self.update(db_obj=user, obj_in={"last_login_at": datetime.utcnow()})


from sqlalchemy.ext.asyncio import AsyncSession
from leodb.models.account.data_source import DataSource
from leodb.schemas.data_source import DataSourceCreate, DataSourceUpdate, DataSourceSchema
from .base_repository import BaseRepository

class DataSourceRepository(BaseRepository[DataSource, DataSourceCreate, DataSourceUpdate, DataSourceSchema]):
    
    def __init__(self, db: AsyncSession, model: type[DataSource]):
        super().__init__(db, model, DataSourceSchema)


from sqlalchemy import MetaData
from sqlalchemy.orm import declarative_base

# Specific metadata for account-related tables.
# This allows us to create all account tables in a specific schema
# without affecting the 'general' tables.
account_metadata = MetaData()

# A specific declarative base for all account-related models.
# Models inheriting from this base will automatically be part of account_metadata.
AccountBase = declarative_base(metadata=account_metadata)